# -*- coding: utf-8 -*-
#IBAN_GUI.py
        
# Klassenunabhaengige Funktionen
# A->10, B->11, ... also Alphabet-Position+10 = ASCII-55
# hinter die Ziffern und 00 fuer die Pruefziffer,
# also 210501700012345678131400
#                        D E 
# Summe modulo 97, Ergebnis von  98 abziehen
# ergibt die Pruefzahl


def buchstabenWert(b): return ord(b)-55

def gibPruefzahl(land,ziffernKette):
    zahl=int(ziffernKette)*1000000
    zahl+=buchstabenWert(land[0])*10000
    zahl+=buchstabenWert(land[1])*100
    zahl=98-(zahl % 97)
    return chr(int(zahl/10)+48)+chr(zahl%10+48)

##print(gibPruefzahl('DE','210501700012345678'))

def pruefe(IBAN):
    zahl=int(IBAN[4:])*1000000
    zahl+=buchstabenWert(IBAN[0])*10000
    zahl+=buchstabenWert(IBAN[1])*100
    zahl+= (ord(IBAN[2])-48)*10
    zahl+= ord(IBAN[3])-48
    return (zahl%97==1)

##print(pruefe('DE68210501700012345678'))
# ---------------GUI-Abschnitt----------------------------------------------
import wx

class IBAN_Frame(wx.Frame):
    """
    Frame fuer das IBAN-Programm
    """
    def __init__(self, parent, title):
        wx.Frame.__init__(self, parent, -1, title,
                          pos=(150, 150), size=(400, 300))

        # Create the menubar
        menuBar = wx.MenuBar()

        # and a menu 
        menu = wx.Menu()
        menu2 =wx.Menu()

        # add an item to the menu, using \tKeyName automatically
        # creates an accelerator, the third param is some help text
        # that will show up in the statusbar
        menu.Append(wx.ID_EXIT, "B&eenden\tAlt-X", "Schliessen der Anwendung")
        menu2.Append(1000, "&Hilfe", "Hilfetext anzeigen")

        self.hilfeText ="Die Eingabe in die Teilfelder kann geprueft werden.\n"
        self.hilfeText+="Bei Fehler wird das Feld rot markiert.\n"
        self.hilfeText+="Bei richtiger IBAN wird das Feld gruen markiert."

        # bind the menu event to an event handler
        self.Bind(wx.EVT_MENU, self.OnTimeToClose, id=wx.ID_EXIT)
        self.Bind(wx.EVT_MENU, self.OnHilfe, id=1000)

        # and put the menu on the menubar
        menuBar.Append(menu, "&Datei")
        menuBar.Append(menu2, "&Hilfe")
        self.SetMenuBar(menuBar)

        self.CreateStatusBar()
        

        # Now create the Panel to put the other controls on.
        panel = wx.Panel(self)

        # ein Label (statische Beschriftung)
        text = wx.StaticText(panel, -1, "IBAN-Programm")
        text.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD))
        text.SetSize(text.GetBestSize())

        # Text-Ctrl - Abschnitt
        self.landTextfeld = wx.TextCtrl(panel, -1, "DE", size=(40, -1))
        self.pruefzifferTextfeld = wx.TextCtrl(panel, -1, "34", size=(40, -1))
        self.blzTextfeld = wx.TextCtrl(panel, -1, "20050550", size=(100, -1))
        self.kontoTextfeld = wx.TextCtrl(panel, -1, "1234567890", size=(120, -1))

        # Button - Abschnitt
        berechneBtn = wx.Button(panel, -1, "berechne")
        berechneBtn.SetToolTip("berechnet die Prüfziffer zur IBAN")
        pruefeBtn = wx.Button(panel, -1, "pruefe")
        pruefeBtn.SetToolTip("prueft die eingegebene IBAN")        
        beendenButton = wx.Button(panel, -1, "Beenden")
        beendenButton.SetToolTip("Button beendet die Anwendung")
        
        # bind the button events to handlers
        self.Bind(wx.EVT_BUTTON, self.OnBerechneButton, berechneBtn)
        self.Bind(wx.EVT_BUTTON, self.OnPruefeButton, pruefeBtn)
        self.Bind(wx.EVT_BUTTON, self.OnTimeToClose, beendenButton)

        # Use a sizer to layout the controls, stacked vertically and with
        # a 10 pixel border around each
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(text, 0, wx.ALL, 10)

        horizontalSizer1 = wx.BoxSizer(wx.HORIZONTAL)
        horizontalSizer1.Add(self.landTextfeld, 0, wx.ALL, 10)
        horizontalSizer1.Add(self.pruefzifferTextfeld, 0, wx.ALL, 10)
        horizontalSizer1.Add(self.blzTextfeld, 0, wx.ALL, 10)
        horizontalSizer1.Add(self.kontoTextfeld, 0, wx.ALL, 10)
        sizer.Add(horizontalSizer1, 0, wx.ALL, 10)

        horizontalSizer2 = wx.BoxSizer(wx.HORIZONTAL)
        horizontalSizer2.Add(berechneBtn, 0, wx.ALL, 10)
        horizontalSizer2.Add(pruefeBtn, 0, wx.ALL, 10)
        horizontalSizer2.Add(beendenButton, 0, wx.ALL, 10)
        sizer.Add(horizontalSizer2, 0, wx.ALL, 10)
        
        panel.SetSizer(sizer)
        panel.Layout()

        # Aktualisieren der Farbe nach erneutem Klick in ein Textfeld
        self.landTextfeld.Bind(wx.EVT_LEFT_DOWN, self.OnTFLeftDown)
        self.pruefzifferTextfeld.Bind(wx.EVT_LEFT_DOWN, self.OnTFLeftDown)
        self.blzTextfeld.Bind(wx.EVT_LEFT_DOWN, self.OnTFLeftDown)
        self.kontoTextfeld.Bind(wx.EVT_LEFT_DOWN, self.OnTFLeftDown)

    def OnTFLeftDown(self, event):
        """Event handler fuer Klick in ein Textfeld."""
        self.landTextfeld.SetBackgroundColour(wx.WHITE)
        self.pruefzifferTextfeld.SetBackgroundColour(wx.WHITE)
        self.blzTextfeld.SetBackgroundColour(wx.WHITE)
        self.kontoTextfeld.SetBackgroundColour(wx.WHITE)
        event.Skip()

    def OnTimeToClose(self, event):
        """Event handler fuer Klick auf Beenden."""
        self.Close()

    def OnPruefeButton(self, event):
        """Event handler für pruefe."""
        iban=self.landTextfeld.GetValue()
        if len(iban)!=2:
            self.landTextfeld.SetBackgroundColour(wx.RED)
            return
        iban+=self.pruefzifferTextfeld.GetValue()
        if len(iban)!=4:
            self.pruefzifferTextfeld.SetBackgroundColour(wx.RED)
            return
        iban+=self.blzTextfeld.GetValue()
        if len(iban)!=12:
            self.blzTextfeld.SetBackgroundColour(wx.RED)
            return
        iban+=self.kontoTextfeld.GetValue()
        if len(iban)!=22:
            self.kontoTextfeld.SetBackgroundColour(wx.RED)
            return
        if pruefe(iban):
            self.pruefzifferTextfeld.SetBackgroundColour(wx.GREEN)
        else:
            self.pruefzifferTextfeld.SetBackgroundColour(wx.RED)

    def OnBerechneButton(self, event):
        """Event handler für berechne."""
        land=self.landTextfeld.GetValue()
        if len(land)!=2:
            self.landTextfeld.SetBackgroundColour(wx.RED)
            return
        blz=self.blzTextfeld.GetValue()
        if len(blz)!=8:
            self.blzTextfeld.SetBackgroundColour(wx.RED)
            return
        konto=self.kontoTextfeld.GetValue()
        if len(konto)!=10:
            self.kontoTextfeld.SetBackgroundColour(wx.RED)
            return
        self.pruefzifferTextfeld.SetValue(gibPruefzahl(land, blz+konto))

    def OnHilfe(self, event):
        self.MeldungAusgeben(self.hilfeText,'Anleitung')

    ### ----- Einfache Meldung ausgeben -----
    def MeldungAusgeben(self, text, wasIst='Meldung'):
        '''Zeigt ein Dialogfenster mit einer Meldung.'''
        dialog = wx.MessageDialog(self, text, wasIst, wx.OK)
        dialog.ShowModal()        
                
# ---------------App-Abschnitt----------------------------------------------
class IBAN_App(wx.App):
    def OnInit(self):
        frame = IBAN_Frame(None, "IBAN - Anwendung")
        self.SetTopWindow(frame)
        frame.Show(True)
        return True
        
app = IBAN_App(redirect=False)
app.MainLoop()

